#################################################################################
# Created       : Martin Foster
# Modified      : 05-May-2003
#################################################################################
#
# Filter - A filtering system that allows for safer handling of posts  
# Copyright (C) 2000-2003  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package eRealms::Filter;					# Package Name

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	my $class = shift;
	my $self  = { };

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


#####################
# SafeHTML 
#
# While the name may be slightly misleading as the HTML can still be malicious to a degree
# this subroutine will aim to make it safer.   Note however, that the only safe HTML is no
# HTML.

sub SafeHTML 	 						# String to handle 
{
	#####################
	# Data members
	my $self    = shift;					# Reference to self
	my $str     = shift;					# String name

	my $count;						# Counter
	my $opn;						# Open tags
	my $tag;						# Singular tag
	my $src;						# Searching value
	
	my @imb;						# Imbalances

	# Tag management
	# Valid tags
	my %allow = (
		A          => 0,				# Link
		ABBR       => 0,				# Abbreviation
		ACRONYM    => 0,				# Acronym
		ADDRESS    => 0,				# Address
		B          => 0,				# Bold
		BIG        => 0,				# One size larger
		BLOCKQUOTE => 0,                                # Block quotation
		BR         => 0,				# Carriage return
		CENTER     => 0,				# Center text
		CITE       => 0,				# Citation
		CODE       => 0,				# Code Sample
		DFN        => 0,				# Defintion
		DIV        => 0,				# Divisions
		EM         => 0,				# Emphasis
		FONT       => 0,				# Set size, face, colour
		HR         => 0,				# Horizontal row
		I          => 0,				# Italics
		P          => 0,				# Paragraph
		Q          => 0,				# Inline quotation
		S          => 0,				# Strike through
		SAMP       => 0,				# Sample
		SMALL      => 0,                                # One size smaller
		STRIKE     => 0,                                # Strike through
		STRONG     => 0,			 	# Bold	
		SUB        => 0,				# Subscript
		SUP        => 0,				# Superscript
		U          => 0,				# Underline
		VAR        => 0					# Variable name
	 );

	# Invalid tags
	my %deny;



	#####################
	# Program area
        # Title filters

	# Strip comments
	$str =~ s/<!-|-->|<COMMENT>|<\/COMMENT>//gsi;

	# Close tags
	while ($str =~ m/<(\w.*?)(\s|>)/gs)
	{
		# Uppercase
		$tag = uc($1);


		# Skips
		next if ($tag =~ /^BR/);
		next if ($tag =~ /^HR/);

		# Track tags
		# Allow
		if (defined($allow{$tag}))
		{
			# Increment for use
			$allow{$tag}++; 
		}

		# Disallow
		else 
		{
			# Ensure definition
			$deny{$tag} = (defined($deny{$tag})) ? $deny{$tag}+1 : 1; 
		}
	}


	# Tag manipulations
	# Filter out invalid code
	foreach $tag (keys(%deny))
	{
		# Erradicate
		# Open and close tag
		$str =~ s/<${tag}.*?>|<\/${tag}>//gsi;
	}


	# Determine need for closing tags
	foreach $tag (keys(%allow))
	{
		# Only if needed
		next if ($allow{$tag} < 1);


		# Set value
		$opn = 0;

		# Count closing tags
		while ($str =~ m/<\/$tag>/gsi)
		{ $opn++; }

		# Add whatever is needed
		for ($count = $opn; $count < $allow{$tag}; $count++)
		{ $str = "$str</$tag>"; }	
	}

	# Imbalances
	# Discovery
	while ($str =~ m/<(\w.*?)>/gs)
	{
		# Assign / Reset
		$tag   = $1;
		$count = 0;

		# Imbalance
		# Verify
		$count++ while ($tag =~ m/"/gs);
		
		# Exist
		if (($count % 2) != 0)
		{
			# Normal
			push(@imb, $tag);


			# Escaped
			# Escape
			$tag = quotemeta($tag);

			# Store
			push(@imb, $tag);
		}
	}

	# Search replace
	while ($tag = shift(@imb))
	{
		# Searching
		$src = shift(@imb);

		# Replace tag
		$str =~ s/<$src>/<$tag\">/gsi;
	}


	# Return
	return $str;
}


#####################
# Strip HTML 
#
# Strip HTML is a harsher addtion to the system.  As this is designed to scrub
# much more of the HTML then the above version.  This is of course to prevent 
# people from adding HTML in things that are useful like a puppet name.

sub StripHTML 	 						# String to handle 
{
	#####################
	# Data members
	my $self    = shift;					# Reference to self
	my $str     = shift;					# String name


	#####################
	# Program area
        # Title filters

	# Html and special
	$str =~ s/<[^>]*>//gs;
	$str =~ s/\&.*?\;//gs;

	# Comments
	$str =~ s/<!-|-->//gs;

	# Return
	return $str;
}


#####################
# Sanity
#
# Will make sure that no single set of characters is longer then a certain amount
# as to prevent the horizontal scroll bar from kicking in.   As a result, hyperlinks
# must be dealt with specially in order to prevent their truncation.

sub MakeSane
{
	#####################
	# Data members
	my $self = shift;					# Self referencing scalar
	my $str  = shift;					# String to look into
	my $num  = shift;					# Number to truncate with

	my $tmp;

	my $cnt = 0;						# Counter
	my $eml;						# Amount of Email
	my $lnk;						# Amount of links

	my $fin;						# Final string
	my $org;						# Original position
	my $pos;						# Position
	my $lnt;						# Determined length

	my @eml;						# Email addresss
	my @lnk; 						# Link holder


	#####################
	# Program area

	# Hyperlink handling
	# Loop and find
	# Hyperlinks
	while ($str =~ /(http:\/\/.*?)(\s|$)/)
	{
		# Assign
		$org = quotemeta($1);
	 	$lnk[$cnt] = $1;			

		# Replace
		$str =~ s/$org/LINK$cnt/gsi;

		# Incremenet
		$cnt++;
	}

	# Assign
	$lnk = $cnt;
	$cnt = 0;


	# Email addresses
	while ($str =~ m/([\w\.\-]*?\@[\w\.\-]*?\..*?)(\s|$)/)
	{
		# Assign
		$org = quotemeta($1);
	 	$eml[$cnt] = $1;			

		# Replace
		$str =~ s/$org/MAIL$cnt/gsi;

		# Incremenet
		$cnt++;
	}

	# Assign
	$eml = $cnt;
	

	# All long strings
	# Loop and find
	while ($str =~ m/(\S+)/gs)
	{
		# Reset
		$pos = 0;

		# Determine need to correct
		if (length($1) > $num)
		{
			# Length of string
			$lnt = length($1);


			# Initial assignmants
			$org =  quotemeta($1);
			$fin =  substr($1, 0, $num);
			$pos += $num;


			# Loop
			# Ensure truncation
			while ($pos < $lnt)
			{
				# Truncate and add space
				$fin = $fin . ' ' . substr($1, $pos, $num);

				# Increment
				$pos += $num;
			}
		
			# Replace values
			$str =~ s/$org/$fin/gs;
		}
	}


	# Re-establish sanity
	# Return hyperlinks
	for ($cnt=0; $cnt < $lnk; $cnt++)
	{
		# Replace with orginal value
		$str =~ s/LINK$cnt/$lnk[$cnt]/gs;
	}	

	# Email addresses
	for ($cnt=0; $cnt < $eml; $cnt++)
	{
		# Replace with orginal value
		$str =~ s/MAIL$cnt/$eml[$cnt]/gs;
	}

	# Return for use
	return $str;
}

1;								# Return true
